from System.Diagnostics import ProcessStartInfo
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from TestStack.White import Application
from robot.api import logger


class ApplicationKeywords(LibraryComponent):
    @keyword
    def launch_application(self, sut_path, args=None):
        """Launches an application.

        ``sut_path`` is the absolute path to the application to launch.

        ``args`` is a string of arguments to use when starting the application (optional).

        Examples:
        | Launch Application | C:/path/to/MyApp.exe | | # Launch without arguments |
        | Launch Application | C:/path/to/MyApp.exe | /o log.txt | # Launch with arguments |
        """
        if args is not None:
            process_start_info = ProcessStartInfo(sut_path)
            process_start_info.Arguments = args
            self.state.app = Application.Launch(process_start_info)
        else:
            self.state.app = Application.Launch(sut_path)

    @keyword
    def attach_application_by_name(self, sut_name):
        """Attaches a running application by name.

        ``sut_name`` is the name of the process.

        Example:
        | Attach Application By Name | UIAutomationTest |
        """
        self.state.app = Application.Attach(sut_name)

    @keyword
    def attach_application_by_id(self, sut_id):
        """Attaches a running application by process id.

        ``sut_id`` is the application process id.

        Example:
        | Attach Application By Id | 12188 |
        """
        self.state.app = Application.Attach(int(sut_id))

    @keyword
    def close_application(self):
        """Closes the attached application."""
        self.state.app.Close()
        self.state.app = None
        self.state.window = None



from TestStack.White.Configuration import CoreAppXmlConfiguration
from TestStack.White.UIItems import DateFormat
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from robot.api import logger
from robot.utils import timestr_to_secs


class WhiteConfigurationKeywords(LibraryComponent):
    @keyword
    def set_white_busy_timeout(self, timeout):
        """Sets BusyTimeout for White and returns original value.

        ``timeout`` is the timeout value as Robot time string.

         Default timeout is 5 seconds.
        """
        CoreAppXmlConfiguration.Instance.BusyTimeout = self._get_timestr_in_milliseconds(timeout)
        logger.info("White Busy Timeout set to" + str(CoreAppXmlConfiguration.Instance.BusyTimeout))
        return self._get_milliseconds_as_timestr(CoreAppXmlConfiguration.Instance.BusyTimeout)

    @keyword
    def get_white_busy_timeout(self):
        """Returns BusyTimeout value of White."""
        return self._get_milliseconds_as_timestr(CoreAppXmlConfiguration.Instance.BusyTimeout)

    @keyword
    def set_white_find_window_timeout(self, timeout):
        """Sets FindWindowTimeout for White and returns original value.

        ``timeout`` is the timeout value as Robot time string.

        Default timeout is 30 seconds.
        """
        CoreAppXmlConfiguration.Instance.FindWindowTimeout = self._get_timestr_in_milliseconds(timeout)
        logger.info("White FindWindowTimeout set to" + str(CoreAppXmlConfiguration.Instance.FindWindowTimeout))
        return self._get_milliseconds_as_timestr(CoreAppXmlConfiguration.Instance.FindWindowTimeout)

    @keyword
    def get_white_find_window_timeout(self):
        """Returns FindWindowTimeout value of White."""
        return self._get_milliseconds_as_timestr(CoreAppXmlConfiguration.Instance.FindWindowTimeout)

    @keyword
    def set_white_double_click_interval(self, interval):
        """Sets DoubleClickInterval for White and returns original value.

        DoubleClickInterval adds delay in double click action between clicks.

        ``interval`` is the interval value as Robot time string.

        Default interval is 0 milliseconds.
        """
        CoreAppXmlConfiguration.Instance.DoubleClickInterval = self._get_timestr_in_milliseconds(interval)
        logger.info("White DoubleClickInterval set to" + str(CoreAppXmlConfiguration.Instance.DoubleClickInterval))
        return self._get_milliseconds_as_timestr(CoreAppXmlConfiguration.Instance.DoubleClickInterval)

    @keyword
    def get_white_double_click_interval(self):
        """Returns DoubleClickInterval value of White."""
        return self._get_milliseconds_as_timestr(CoreAppXmlConfiguration.Instance.DoubleClickInterval)

    def _get_timestr_in_milliseconds(self, time_string):
        return timestr_to_secs(time_string) * 1000

    def _get_milliseconds_as_timestr(self, milliseconds):
        return "{} milliseconds".format(milliseconds)
